#!/usr/bin/perl

#
# Copyright (C) 2010 FromDual
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; version 2 of the License.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

use strict;
use warnings;

# To avoid File::basedir
# basedir is known in perl 5.6 so we can use it!
my $lMyName = $0;
my $file = $ARGV[0];

$lMyName =~ s/^.*\///g;
my $lMyNameBase = $lMyName;
$lMyNameBase =~ s/(\..*)?$//g;


my $basedir;
BEGIN {
  $basedir = '/usr/local/mysql_performance_monitor';
}
use lib $basedir."/lib";

use FromDualMySQLagent;
use FromDualMySQLagent ':stooges';

my $lConfFile = '';
if ( defined($file) && ("$file" ne '') ) {
  $lConfFile = "$file";
}
else {
  $lConfFile = $basedir . '/etc/' . $lMyNameBase . '.conf';
}

#basename
our $gDomain = 'FromDual.MySQL';
my @configuration = &FromDualMySQLagent::readConfigurationFile($lConfFile);
my @sections = &FromDualMySQLagent::getSections(@configuration);
my %defaults = &FromDualMySQLagent::getDefaultParameter(\@configuration, 'default');

# This is the new way to do it if it works
# remove all refs to make code easier...

our $gLogFile   = $defaults{'LogFile'};
our $gCacheFile = "/tmp/FromDualAgentCacheFile.dat";
our $gDebug     = $defaults{'Debug'};

if ( $defaults{'Debug'} >= INFO ) {
  &FromDualMySQLagent::log($defaults{'LogFile'}, INFO, "FromDual performance monitor for MySQL run started.");
  &FromDualMySQLagent::log($defaults{'LogFile'}, INFO, "Read configuratoin from $lConfFile.");
  &FromDualMySQLagent::log($defaults{'LogFile'}, INFO, 'Sections found: ' . join(', ', @sections));
  &FromDualMySQLagent::log($defaults{'LogFile'}, INFO, "Reading default section.");
}
if ( $defaults{'Debug'} >= INFO ) {

  foreach my $p ( keys %defaults) {
    &FromDualMySQLagent::log($defaults{'LogFile'}, INFO, "$p - $defaults{$p}");
  }
}

foreach my $section ( @sections ) {

  if ( $section ne 'default' ) {

    my %parameter = ();
    %parameter = &FromDualMySQLagent::getParameter(\@configuration, $section);
    $gDebug = $parameter{'Debug'};

    if ( $parameter{'Debug'} >= INFO ) {
      &FromDualMySQLagent::log($parameter{'LogFile'}, INFO, "Processing section $section now...");
    }

    if ( $parameter{'Debug'} == DBG ) {

      &FromDualMySQLagent::log($parameter{'LogFile'}, DBG, "Parameters for section: $section");
      foreach my $p ( keys %parameter ) {
        &FromDualMySQLagent::log($parameter{'LogFile'}, DBG, "$p - $parameter{$p}");
      }
    }

    if ( $parameter{'Disabled'} eq 'true' ) {
      if ( $parameter{'Debug'} >= INFO ) {
        &FromDualMySQLagent::log($parameter{'LogFile'}, INFO, "Skipping section: $section because it is disabled.");
      }
      next;
    }

    if ( $parameter{'Debug'} >= INFO ) {
      &FromDualMySQLagent::log($parameter{'LogFile'}, INFO, "Modules for section $section: " . $parameter{'Modules'});
    }

    my @modules = split(/\s+/, $parameter{'Modules'});
    foreach my $module ( @modules ) {

      $parameter{'Domain'} = 'FromDual.MySQL.' . $module;

      if ( ($parameter{'Debug'} >= INFO) ) {
        &FromDualMySQLagent::log($parameter{'LogFile'}, INFO, "Processing module $module for section $section now...");
      }

      if ( $module eq 'process' ) {
        eval {
          require FromDualMySQLprocess;
        };

        if ( $@ ) {
          &FromDualMySQLagent::log($parameter{'LogFile'}, ERR, "Module FromDualMySQLprocess does not exist.");
          foreach my $key ( $@ ) {
            &FromDualMySQLagent::log($parameter{'LogFile'}, DBG, $key);
          }
        }
        else {
          &FromDualMySQLprocess::processProcessInformation(\%parameter);
        }
      }
      elsif ( $module eq 'ndb' ) {
        eval {
          require FromDualMySQLndb;
        };

        if ( $@ ) {
          &FromDualMySQLagent::log($parameter{'LogFile'}, ERR, "Module FromDualMySQLndb does not exist.");
          foreach my $key ( $@ ) {
            &FromDualMySQLagent::log($parameter{'LogFile'}, DBG, $key);
          }
        }
        else {
          &FromDualMySQLndb::processNdbInformation(\%parameter);
        }
      }
      elsif ( $module eq 'master' ) {
        eval {
          require FromDualMySQLmaster;
        };

        if ( $@ ) {
          &FromDualMySQLagent::log($parameter{'LogFile'}, ERR, "Module FromDualMySQLmaster does not exist.");
          foreach my $key ( $@ ) {
            &FromDualMySQLagent::log($parameter{'LogFile'}, DBG, $key);
          }
        }
        else {
          &FromDualMySQLmaster::processMasterInformation(\%parameter);
        }
      }
      elsif ( $module eq 'slave' ) {
        eval {
          require FromDualMySQLslave;
        };

        if ( $@ ) {
          &FromDualMySQLagent::log($parameter{'LogFile'}, ERR, "Module FromDualMySQLslave does not exist.");
          foreach my $key ( $@ ) {
            &FromDualMySQLagent::log($parameter{'LogFile'}, DBG, $key);
          }
        }
        else {
          &FromDualMySQLslave::processSlaveInformation(\%parameter);
        }
      }
      elsif ( $module eq 'mysql' ) {
        eval {
          require FromDualMySQLmysql;
        };

        if ( $@ ) {
          &FromDualMySQLagent::log($parameter{'LogFile'}, ERR, "Module FromDualMySQLmysql does not exist.");
          foreach my $key ( $@ ) {
            &FromDualMySQLagent::log($parameter{'LogFile'}, DBG, $key);
          }
        }
        else {
          &FromDualMySQLmysql::processMysqlInformation(\%parameter);
        }
      }
      elsif ( $module eq 'myisam' ) {
        eval {
          require FromDualMySQLmyisam;
        };

        if ( $@ ) {
          &FromDualMySQLagent::log($parameter{'LogFile'}, ERR, "Module FromDualMySQLmyisam does not exist.");
          foreach my $key ( $@ ) {
            &FromDualMySQLagent::log($parameter{'LogFile'}, DBG, $key);
          }
        }
        else {
          &FromDualMySQLmyisam::processMyisamInformation(\%parameter);
        }
      }
      elsif ( $module eq 'innodb' ) {
        eval {
          require FromDualMySQLinnodb;
        };

        if ( $@ ) {
          &FromDualMySQLagent::log($parameter{'LogFile'}, ERR, "Module FromDualMySQLpbxt does not exist.");
          foreach my $key ( $@ ) {
            &FromDualMySQLagent::log($parameter{'LogFile'}, DBG, $key);
          }
        }
        else {
          &FromDualMySQLinnodb::processInnodbInformation(\%parameter);
        }
      }
      elsif ( $module eq 'pbxt' ) {
        eval {
          require FromDualMySQLpbxt;
        };

        if ( $@ ) {
          &FromDualMySQLagent::log($parameter{'LogFile'}, ERR, "Module FromDualMySQLpbxt does not exist.");
          foreach my $key ( $@ ) {
            &FromDualMySQLagent::log($parameter{'LogFile'}, DBG, $key);
          }
        }
        else {
          &FromDualMySQLpbxt::processPbxtInformation(\%parameter);
        }
      }
      elsif ( $module eq 'xtradb' ) {
        eval {
          require FromDualMySQLxtradb;
        };

        if ( $@ ) {
          &FromDualMySQLagent::log($parameter{'LogFile'}, ERR, "Module FromDualMySQLxtradb does not exist.");
          foreach my $key ( $@ ) {
            &FromDualMySQLagent::log($parameter{'LogFile'}, DBG, $key);
          }
        }
        else {
          &FromDualMySQLxtradb::processXtradbInformation(\%parameter);
        }
      }
      elsif ( $module eq 'aria' ) {
        eval {
          require FromDualMySQLaria;
        };

        if ( $@ ) {
          &FromDualMySQLagent::log($parameter{'LogFile'}, ERR, "Module FromDualMySQLaria does not exist.");
          foreach my $key ( $@ ) {
            &FromDualMySQLagent::log($parameter{'LogFile'}, DBG, $key);
          }
        }
        else {
          &FromDualMySQLaria::processAriaInformation(\%parameter);
        }
      }
      elsif ( $module eq 'server' ) {
        eval {
          require FromDualMySQLserver;
        };

        if ( $@ ) {
          &FromDualMySQLagent::log($parameter{'LogFile'}, ERR, "Module FromDualMySQLserver does not exist.");
          foreach my $key ( $@ ) {
            &FromDualMySQLagent::log($parameter{'LogFile'}, DBG, $key);
          }
        }
        else {
          &FromDualMySQLserver::processServerInformation(\%parameter);
        }
      }
      elsif ( $module eq 'drbd' ) {
        eval {
          require FromDualMySQLdrbd;
        };

        if ( $@ ) {
          &FromDualMySQLagent::log($parameter{'LogFile'}, ERR, "Module FromDualMySQLdrbd does not exist.");
          foreach my $key ( $@ ) {
            &FromDualMySQLagent::log($parameter{'LogFile'}, DBG, $key);
          }
        }
        else {
          our %gParameter = %parameter;
          &FromDualMySQLdrbd::processDrbdInformation();
        }
      }
      else {
        &FromDualMySQLagent::log($parameter{'LogFile'}, ERR, "Module $module does not exist.");
      }
    }
  }
}

print "1\n";
exit 0;
