package FromDualMySQLserver;

#
# Copyright (C) 2011 FromDual
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; version 2 of the License.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

use strict;
use warnings;
use FromDualMySQLagent ':stooges';

sub processServerInformation
{
  my $parameter_ref = shift;

  my %hServerStatus;

  my @aServerStatusToSend = (
    'system.cpu.ctxt', 'system.cpu.interrupts', 'system.cpu.processes', 'system.cpu.procs_running', 'system.cpu.procs_blocked'
  , 'system.cpu.cpu_user', 'system.cpu.cpu_nice', 'system.cpu.cpu_system', 'system.cpu.cpu_idle', 'system.cpu.cpu_iowait'
  , 'system.cpu.cpu_irq', 'system.cpu.cpu_softirq', 'system.cpu.cpu_steal', 'system.cpu.cpu_guest', 'system.cpu.cpu_unknown'
  );

  &FromDualMySQLagent::initValues(\%hServerStatus, \@aServerStatusToSend);

  # /proc/stat

  open STAT, '<', '/proc/stat' or dir $!;
  while ( <STAT> ) {
    chomp $_;
    if ( $_ =~ /^ctxt\s*(\d*)/ ) {
      $hServerStatus{'system.cpu.ctxt'} = $1;
    }
    elsif ( $_ =~ /^processes\s*(\d*)/ ) {
      $hServerStatus{'system.cpu.processes'} = $1;
    }
    elsif ( $_ =~ /^intr\s*(\d*)/ ) {
      $hServerStatus{'system.cpu.interrupts'} = $1;
    }
    elsif ( $_ =~ /^procs_running\s*(\d*)/ ) {
      $hServerStatus{'system.cpu.procs_running'} = $1;
    }
    elsif ( $_ =~ /^procs_blocked\s*(\d*)/ ) {
      $hServerStatus{'system.cpu.procs_blocked'} = $1;
    }
    elsif ( $_ =~ /^cpu\s+(\d*)\s+(\d+)\s+(\d+)\s+(\d+)\s+(\d+)\s+(\d+)\s+(\d+)\s+(\d+)\s+(\d+)\s+(\d+)/ ) {
      $hServerStatus{'system.cpu.cpu_user'} = $1;
      $hServerStatus{'system.cpu.cpu_nice'} = $2;
      $hServerStatus{'system.cpu.cpu_system'} = $3;
      $hServerStatus{'system.cpu.cpu_idle'} = $4;
      $hServerStatus{'system.cpu.cpu_iowait'} = $5;
      $hServerStatus{'system.cpu.cpu_irq'} = $6;
      $hServerStatus{'system.cpu.cpu_softirq'} = $7;
      $hServerStatus{'system.cpu.cpu_steal'} = $8;
      $hServerStatus{'system.cpu.cpu_guest'} = $9;
      $hServerStatus{'system.cpu.cpu_unknown'} = $10;
    }
  } 
  close(STAT);

  # /proc/diskstats

  open STAT, '<', '/proc/diskstats' or dir $!;
  while ( <STAT> ) {
    chomp $_;



    if ( $_ =~ /^\s+\d+\s+\d+\s+(\w+)\s+(\d+)\s+(\d+)\s+(\d+)\s+(\d+)\s+(\d+)\s+(\d+)\s+(\d+)\s+(\d+)\s+(\d+)\s+(\d+)\s+(\d+)/ ) {

      # skip ram and loop
      if ( ($1 =~ /^ram/) || ($1 =~ /^loop/) ) {
        next;
      }

      push(@aServerStatusToSend, 'io.device.reads_completed[' . $1 . ']');
      $hServerStatus{'io.device.reads_completed[' . $1 . ']'} = $2;
      push(@aServerStatusToSend, 'io.device.reads_merged[' . $1 . ']');
      $hServerStatus{'io.device.reads_merged[' . $1 . ']'} = $3;
      push(@aServerStatusToSend, 'io.device.sectors_read[' . $1 . ']');
      $hServerStatus{'io.device.sectors_read[' . $1 . ']'} = $4;
      push(@aServerStatusToSend, 'io.device.ms_spent_reading[' . $1 . ']');
      $hServerStatus{'io.device.ms_spent_reading[' . $1 . ']'} = $5;
      push(@aServerStatusToSend, 'io.device.writes_completed[' . $1 . ']');
      $hServerStatus{'io.device.writes_completed[' . $1 . ']'} = $6;
      push(@aServerStatusToSend, 'io.device.writes_merged[' . $1 . ']');
      $hServerStatus{'io.device.writes_merged[' . $1 . ']'} = $7;
      push(@aServerStatusToSend, 'io.device.sectors_written[' . $1 . ']');
      $hServerStatus{'io.device.sectors_written[' . $1 . ']'} = $8;
      push(@aServerStatusToSend, 'io.device.ms_spent_writing[' . $1 . ']');
      $hServerStatus{'io.device.ms_spent_writing[' . $1 . ']'} = $9;
      push(@aServerStatusToSend, 'io.device.io_in_progress[' . $1 . ']');
      $hServerStatus{'io.device.io_in_progress[' . $1 . ']'} = $10;
      push(@aServerStatusToSend, 'io.device.ms_spent_doing_io[' . $1 . ']');
      $hServerStatus{'io.device.ms_spent_doing_io[' . $1 . ']'} = $11;

      # This field is incremented at each I/O start, I/O completion, I/O
      # merge, or read of these stats by the number of I/Os in progress
      # (field 9) times the number of milliseconds spent doing I/O since the
      # last update of this field.  This can provide an easy measure of both
      # I/O completion time and the backlog that may be accumulating.
      push(@aServerStatusToSend, 'io.device.wighted_nbr_ms_spint_doin_io[' . $1 . ']');
      $hServerStatus{'io.device.wighted_nbr_ms_spent_doing_io[' . $1 . ']'} = $12;
    }
  }
  close(STAT);

  # Do some post calculations

  # $hServerStatus{'xxx'} = $hServerStatus{'xxx'} == 0 ? 0 : $hServerStatus{'xxx'} / $hServerStatus{'xxx'};

  # NO domain in this case!
  &FromDualMySQLagent::sendData(\%$parameter_ref, \%hServerStatus, \@aServerStatusToSend, '');
}

1;
__END__
